document.addEventListener('DOMContentLoaded', () => {
	document.querySelectorAll('#search').forEach(p => {
		let [input, button] = p.children;
		button.onclick = e => {
			if (input.value && input.offsetWidth > 50) {
				return true; // bubble to submit
			}
			// make visible first
			input.focus();
			e.preventDefault();
			return false;
		};
	});

	document.querySelectorAll('.multiinput > input[id]').forEach(el => {
		el.oninput = e => {
			if (e.target.value == '') return;
			// insert another empty input element option
			let add = e.target.cloneNode(true);
			add.value = '';
			add.oninput = e.target.oninput;
			e.target.parentNode.appendChild(add);
			e.target.oninput = undefined;
			e.target.removeAttribute('id');
		};
	});

	let wpinput = document.getElementById('wptitle');
	if (wpinput) {
		let wpbutton = wpinput.parentNode.appendChild(document.createElement('button'));
		wpbutton.type = 'button';
		wpbutton.append('Download');
		wpbutton.onclick = () => {
			let wptitle = wpinput.value || document.getElementById('form').value;
			let wplang = document.getElementById('lang').value;
			if (wplang == 'la') wplang = 'en'; // most likely presence of scientific names
			let wpapi = `https://${wplang}.wikipedia.org/w/api.php`;
			let wppage = wpapi+'?action=parse&format=json&origin=*&prop=text|langlinks&page='+wptitle;
			fetch(wppage).then(res => res.json()).then(json => {
				if (json.error) throw `error returned: ${json.error.info}`;
				wpinput.value = json.parse.title;

				let wptext = json.parse.text['*'];
				let transrow = document.getElementById('trans-la');
				if (transrow && !transrow.value && wptext) {
					const binom = wptext.match(/ class="binomial">.*?<i>(.*?)<\/i>/);
					if (binom) {
						transrow.value = binom[1]
					}
				}

				// translations from language links
				let wplangs = json.parse.langlinks;
				if (wplangs) wplangs.forEach(wptrans => {
					let transrow = document.getElementById('trans-' + wptrans.lang);
					if (!transrow || transrow.value) return;
					transrow.value = wptrans['*'].replace(/([^,(]*).*/, (link, short) => {
						return short.toLocaleLowerCase(wptrans.lang).trimEnd() + ' [' + link + ']';
					});
				});

				// copy first paragraph to story
				let storyinput = document.getElementById('story');
				if (storyinput && !storyinput.value && wptext) {
					storyinput.value = wptext
						.replace(/<h2.*/s, '') // prefix
						.replace(/<table.*?<\/table>/sg, '') // ignore infobox
						.match(/<p>(.*?)<\/p>/s)[0] // first paragraph
						.replace(/<[^>]*>/g, '') // strip html tags
				}

				// list images in article html
				let imginput = document.getElementById('source');
				if (!imginput || imginput.value) return;
				let wpimages = wptext.match(/<img\s[^>]+>/g);
				let wpselect = wpinput.parentNode.appendChild(document.createElement('ul'));
				wpselect.className = 'popup';
				wpimages.forEach(img => {
					let selectitem = wpselect.appendChild(document.createElement('li'));
					selectitem.insertAdjacentHTML('beforeend', img);
					selectitem.onclick = e => {
						let imgsrc = e.target.src
							.replace(/^(?=\/\/)/, 'https:')
							.replace(/\/thumb(\/.+)\/[^\/]+$/, '$1');
						imginput.value = imgsrc;
						wpselect.remove();
						return false;
					};
				});
			}).catch(error => alert(error));
			return false;
		};
		wpbutton = wpinput.parentNode.appendChild(document.createElement('button'));
		wpbutton.type = 'button';
		wpbutton.append('Visit');
		wpbutton.onclick = () => {
			let wptitle = wpinput.value || document.getElementById('form').value;
			let wplang = document.getElementById('lang').value;
			let wpurl =
				wplang == 'la' ? `https://species.wikimedia.org/wiki/${wptitle}` :
				`https://${wplang}.wikipedia.org/wiki/${wptitle}`;
			window.open(wpurl, 'sheet-wikipedia').focus();
			return false;
		};
	}

	let imgpreview = document.getElementById('sourcepreview');
	if (imgpreview) {
		let imginput = document.getElementById('source');
		imginput.parentNode.parentNode.append(imgpreview); // separate row
		let previewbutton = imginput.parentNode.appendChild(document.createElement('button'));
		previewbutton.type = 'button';
		previewbutton.append('View');
		previewbutton.onclick = () => {
			previewbutton.childNodes[0].nodeValue = imgpreview.hidden ? 'Hide' : 'View';
			imgpreview.hidden = !imgpreview.hidden;
		};
	}

	let thumbpreview = document.getElementById('convertpreview');
	if (thumbpreview && imgpreview) {
		thumbpreview.onclick = e => {
			const cropinput = document.getElementById('crop32');
			let border = { width: 600, height: 400 };
			let crop = new Croppie(thumbpreview, {
				boundary: { width: border.width * 1.3, height: border.height * 1.2 },
				viewport: border,
				update: e => {
					cropinput.value = e.points.map((pos, axis) => {
						Math.round(1000 * pos / (axis % 2 ? border.height : border.width))
					}).join(',');
				},
			});
			crop.bind({
				url: imgpreview.src,
				points: cropinput.value.split(/[^0-9]/).map((rel, axis) => {
					return rel * (axis % 2 ? border.height : border.width) / 1000
				}),
			});
			crop.elements.overlay.addEventListener('dblclick', e => {
				crop.destroy();
			});
		};
	}

	let translist = document.getElementById('trans');
	if (translist) {
		let langoptions = Array.prototype.filter.call(document.getElementById('lang').options, opt => {
			if (document.getElementById('trans-' + opt.value)) return;
			if (document.getElementById('lang').value == opt.value) return;
			return true;
		});
		if (!langoptions.length) return;

		let transadd = translist.appendChild(document.createElement('li'));
		let transselect = transadd.appendChild(document.createElement('select'));
		transselect.appendChild(document.createElement('option'));
		for (let langoption of langoptions) {
			let transoption = document.createElement('option');
			transoption.value = langoption.value;
			transoption.append(langoption.label);
			transselect.appendChild(transoption);
		}
		transselect.onchange = e => {
			let inputlang = e.target.selectedOptions[0];
			let transadded = translist.insertBefore(document.createElement('li'), transadd);
			let translabel = transadded.appendChild(document.createElement('label'));
			translabel.append(inputlang.label.replace(/ (.+)/, ' ')); //TODO title = $1
			let transinput = transadded.appendChild(document.createElement('input'));
			transinput.name = 'trans-'+inputlang.value;
			translabel.setAttribute('for', transinput.id = transinput.name);
			inputlang.remove();
			if (e.target.length <= 1) e.target.remove();
			transinput.focus();
		};
	}
});
